@@ -158,14 +158,14 @@ std::vector<CutlassTileConfig> get_candidate_tiles(
               CutlassTileConfig::CtaShape256x128x64_WarpShape64x64x64};
     case CutlassGemmType::Fp8:
       if (config_type_param & CutlassGemmConfig::GROUPED_GEMM) {
-        if (sm == 89 || sm >= 120) {
-          return {CutlassTileConfig::CtaShape16x256x128_WarpShape16x64x128,
-                  CutlassTileConfig::CtaShape32x128x64_WarpShape32x32x64,
+        if (sm == 89 || sm == 120 || sm == 121) {
+          return {CutlassTileConfig::CtaShape32x128x64_WarpShape32x32x64,
                   CutlassTileConfig::CtaShape64x128x64_WarpShape64x32x64,
                   CutlassTileConfig::CtaShape64x64x128_WarpShape32x64x64,
                   CutlassTileConfig::CtaShape128x64x64_WarpShape64x32x64,
                   CutlassTileConfig::CtaShape128x256x64_WarpShape64x64x64,
-                  CutlassTileConfig::CtaShape256x128x64_WarpShape64x64x64};
+                  CutlassTileConfig::CtaShape256x128x64_WarpShape64x64x64,
+                  CutlassTileConfig::CtaShape16x256x128_WarpShape16x64x128};
         } else {
           // no valid ampere style fp8 configs for sm90
           return {};
@@ -264,6 +264,119 @@ bool sm90_supports_mcast_along_n(CutlassTileConfigSM90 const tile) {
 #endif
 }
 
+std::vector<CutlassGemmConfig> get_candidate_configs_sm100_dynamic_cluster_shape(
+    int sm, CutlassGemmConfig::CandidateConfigTypeParam const config, EpilogueScheduleType schedule,
+    ClusterShape const dynamic_cluster_shape, ClusterShape const fallback_cluster_shape) {
+  auto cluster1sm = ClusterShape::ClusterShape_1x1x1;
+  auto cluster2sm = ClusterShape::ClusterShape_2x1x1;
+  bool supports_2sm = dynamic_cluster_shape == ClusterShape::Undefined ||
+                      std::get<0>(enum_to_shape_tuple(dynamic_cluster_shape)) % 2 == 0;
+
+  std::vector<CutlassGemmConfig> candidate_configs;
+  if ((config & CutlassGemmConfig::FP4_ONLY) != 0) {
+    if (sm == 100) {
+      if (schedule != EpilogueScheduleType::TMA) return {};
+      candidate_configs.push_back(CutlassGemmConfig{
+          CutlassTileConfigSM100::CtaShape128x64x128B, MainloopScheduleType::AUTO, schedule,
+          cluster1sm, dynamic_cluster_shape, fallback_cluster_shape, sm});
+      if (supports_2sm) {
+        candidate_configs.push_back(CutlassGemmConfig{
+            CutlassTileConfigSM100::CtaShape128x64x128B, MainloopScheduleType::AUTO, schedule,
+            cluster2sm, dynamic_cluster_shape, fallback_cluster_shape, sm});
+      }
+    }
+
+    candidate_configs.push_back(
+        CutlassGemmConfig{CutlassTileConfigSM100::CtaShape128x128x128B, MainloopScheduleType::AUTO,
+                          schedule, cluster1sm, dynamic_cluster_shape, fallback_cluster_shape, sm});
+    candidate_configs.push_back(
+        CutlassGemmConfig{CutlassTileConfigSM100::CtaShape128x256x128B, MainloopScheduleType::AUTO,
+                          schedule, cluster1sm, dynamic_cluster_shape, fallback_cluster_shape, sm});
+    if (supports_2sm) {
+      candidate_configs.push_back(CutlassGemmConfig{
+          CutlassTileConfigSM100::CtaShape128x128x128B, MainloopScheduleType::AUTO, schedule,
+          cluster2sm, dynamic_cluster_shape, fallback_cluster_shape, sm});
+      candidate_configs.push_back(CutlassGemmConfig{
+          CutlassTileConfigSM100::CtaShape128x256x128B, MainloopScheduleType::AUTO, schedule,
+          cluster2sm, dynamic_cluster_shape, fallback_cluster_shape, sm});
+    }
+    return candidate_configs;
+  }
+
+  std::vector<std::pair<CutlassTileConfigSM100, ClusterShape>> tile_configs{
+      {CutlassTileConfigSM100::CtaShape128x128x128B, cluster1sm},
+      {CutlassTileConfigSM100::CtaShape128x256x128B, cluster1sm},
+      {CutlassTileConfigSM100::CtaShape128x32x128B, cluster1sm},
+      {CutlassTileConfigSM100::CtaShape64x64x128B, cluster1sm},
+      {CutlassTileConfigSM100::CtaShape64x32x128B, cluster1sm},
+      {CutlassTileConfigSM100::CtaShape64x128x128B, cluster1sm},
+      {CutlassTileConfigSM100::CtaShape64x256x128B, cluster1sm},
+      {CutlassTileConfigSM100::CtaShape128x64x128B, cluster1sm},
+  };
+
+  if (supports_2sm) {
+    tile_configs.push_back({CutlassTileConfigSM100::CtaShape64x128x128B, cluster2sm});
+    tile_configs.push_back({CutlassTileConfigSM100::CtaShape64x256x128B, cluster2sm});
+    tile_configs.push_back({CutlassTileConfigSM100::CtaShape64x64x128B, cluster2sm});
+    tile_configs.push_back({CutlassTileConfigSM100::CtaShape128x64x128B, cluster2sm});
+    tile_configs.push_back({CutlassTileConfigSM100::CtaShape128x128x128B, cluster2sm});
+    tile_configs.push_back({CutlassTileConfigSM100::CtaShape128x256x128B, cluster2sm});
+  }
+
+  if (config & CutlassGemmConfig::FP8_ONLY) {
+    tile_configs.push_back({CutlassTileConfigSM100::CtaShape128x16x128B, cluster1sm});
+    // TODO: re-enable when handled by the MoE GEMM dispatch
+    // tile_configs.push_back({ CutlassTileConfigSM100::CtaShape128x8x256B,
+    // ClusterShape::ClusterShape_1x1x1 });
+  }
+
+  for (auto [tile, cluster] : tile_configs) {
+    CutlassGemmConfig config{tile,    MainloopScheduleType::AUTO, schedule,
+                             cluster, dynamic_cluster_shape,      fallback_cluster_shape,
+                             sm};
+    candidate_configs.push_back(config);
+  }
+  return candidate_configs;
+}
+
+std::vector<CutlassGemmConfig> get_candidate_configs_sm100(
+    CutlassGemmConfig::CandidateConfigTypeParam const config, int sm) {
+#ifdef FAST_BUILD
+  // Fast build disables all configs except this one for SM100
+  return {CutlassGemmConfig{CutlassTileConfigSM100::CtaShape128x128x128B,
+                            MainloopScheduleType::AUTO, EpilogueScheduleType::TMA,
+                            ClusterShape::ClusterShape_1x1x1, ClusterShape::Undefined,
+                            ClusterShape::Undefined, sm}};
+#else
+  if (config & CutlassGemmConfig::GROUPED_GEMM) {
+    std::vector<CutlassGemmConfig> candidate_configs;
+    for (auto schedule : {EpilogueScheduleType::TMA, EpilogueScheduleType::NO_SMEM}) {
+      // TODO The tactic profiling is a bit long with all of these shapes enabled
+      //   Shape 4x4x1 shapes do not seem to give better performance in the cases I tested so we
+      //   disable it here
+      auto cluster_shapes = {
+          ClusterShape::ClusterShape_1x1x1, ClusterShape::ClusterShape_4x1x1,
+          ClusterShape::ClusterShape_4x2x1 /*, ClusterShape::ClusterShape_4x4x1*/};
+      for (auto cluster_shape : cluster_shapes) {
+        auto fallback_cluster_shape = cluster_shape == ClusterShape::ClusterShape_1x1x1
+                                          ? ClusterShape::ClusterShape_1x1x1
+                                          : ClusterShape::ClusterShape_2x1x1;
+        auto configs = get_candidate_configs_sm100_dynamic_cluster_shape(
+            sm, config, schedule, cluster_shape, fallback_cluster_shape);
+        candidate_configs.insert(candidate_configs.end(), configs.begin(), configs.end());
+      }
+
+      auto configs = get_candidate_configs_sm100_dynamic_cluster_shape(
+          sm, config, schedule, ClusterShape::Undefined, ClusterShape::Undefined);
+      candidate_configs.insert(candidate_configs.end(), configs.begin(), configs.end());
+    }
+    return candidate_configs;
+  } else {
+    TLLM_THROW("Not Implemented: SM100 GEMM candidates have not been defined.");
+  }
+#endif
+}
+
 std::vector<CutlassGemmConfig> get_candidate_configs_sm90(
     CutlassGemmConfig::CandidateConfigTypeParam const config) {
   auto tiles = get_candidate_tiles_sm90(config);
@@ -330,7 +443,7 @@ std::vector<CutlassGemmConfig> get_candidate_configs_sm90(
   return candidate_configs;
 }
 
-std::vector<CutlassGemmConfig> get_candidate_configs_sm100(
+/*std::vector<CutlassGemmConfig> get_candidate_configs_sm100(
     CutlassGemmConfig::CandidateConfigTypeParam const config) {
 #ifdef FAST_BUILD
   // Fast build disables all configs except this one for SM100
@@ -413,7 +526,7 @@ std::vector<CutlassGemmConfig> get_candidate_configs_sm100(
     TLLM_THROW("Not Implemented: SM100 GEMM candidates have not been defined.");
   }
 #endif
-}
+}*/
 
 std::vector<CutlassGemmConfig> get_candidate_configs_sm110(
     CutlassGemmConfig::CandidateConfigTypeParam const config) {
@@ -538,7 +651,7 @@ std::vector<CutlassGemmConfig> get_candidate_configs(
     return get_candidate_configs_sm110(config_type_param);
   }
   if (sm >= 100 && sm < 120 && (config_type_param & CutlassGemmConfig::BLACKWELL)) {
-    return get_candidate_configs_sm100(config_type_param);
+    return get_candidate_configs_sm100(config_type_param, sm);
   }
   if (sm >= 120 && (config_type_param & CutlassGemmConfig::BLACKWELL)) {
     return get_candidate_configs_sm120(config_type_param);
