@@ -67,7 +67,7 @@
 #include "tensorrt_llm/kernels/cutlass_kernels/cutlass_heuristic.h"
 #include "tensorrt_llm/kernels/cutlass_kernels/cutlass_type_conversion.h"
 
-namespace tensorrt_llm::kernels::cutlass_kernels {
+namespace tensorrt_llm::kernels::cutlass_kernels_oss {
 
 // ============================= Variable batched Gemm things ===========================
 template <typename T, typename WeightType, typename GemmOutputType, typename arch,
@@ -106,9 +106,10 @@ struct genericMoeGemmKernelLauncher {
 
     // The cutlass type for the input elements. This is needed to convert to cutlass::half_t if
     // necessary.
-    using ElementType = typename TllmToCutlassTypeAdapter<T>::type;
-    using CutlassGemmOutputType = typename TllmToCutlassTypeAdapter<GemmOutputType>::type;
-    using CutlassWeightType = typename TllmToCutlassTypeAdapter<WeightType>::type;
+    using ElementType = typename cutlass_kernels::TllmToCutlassTypeAdapter<T>::type;
+    using CutlassGemmOutputType =
+        typename cutlass_kernels::TllmToCutlassTypeAdapter<GemmOutputType>::type;
+    using CutlassWeightType = typename cutlass_kernels::TllmToCutlassTypeAdapter<WeightType>::type;
     if (!inputs.use_fused_moe) {
       // We need separate config for each architecture since we will target different tensorcore
       // instructions. For float, we do not target TCs.
@@ -213,9 +214,9 @@ struct genericMoeGemmKernelLauncher {
                                                                               // support fp16 or
                                                                               // bf16)
     {
-      sm80_generic_fused_moe_gemm_kernelLauncher<ElementType, CutlassWeightType,
-                                                 ThreadblockShape::kM, ThreadblockShape::kN,
-                                                 ThreadblockShape::kK, Stages, EpilogueTag>(
+      tensorrt_llm::kernels::cutlass_kernels_oss::sm80_generic_fused_moe_gemm_kernelLauncher<
+          ElementType, CutlassWeightType, ThreadblockShape::kM, ThreadblockShape::kN,
+          ThreadblockShape::kK, Stages, EpilogueTag>(
           reinterpret_cast<ElementType const*>(inputs.A),
           reinterpret_cast<CutlassWeightType const*>(inputs.B),
           reinterpret_cast<ElementType const*>(inputs.biases), inputs.bias_is_broadcast,
@@ -254,18 +255,19 @@ static void dispatch(GroupedGemmInput<T, WeightType, GemmOutputType, GemmOutputT
   if constexpr ((Stages == 2 || Arch::kMinComputeCapability >= 80) &&
                 (!isFp8 || std::is_same_v<Arch, cutlass::arch::Sm89>) && !isFp4) {
     // dispatch for quant op type
-    auto* launcher = kernels::cutlass_kernels::genericMoeGemmKernelLauncher<
+    auto* launcher = tensorrt_llm::kernels::cutlass_kernels_oss::genericMoeGemmKernelLauncher<
         T, WeightType, GemmOutputType, Arch, cutlass::WeightOnlyQuantOp::UNDEFINED, EpilogueTag,
         ThreadblockShape, WarpShape, Stages>::call;
     if (!std::is_same_v<WeightType, T> && inputs.groupwise_quant_group_size > 0) {
-      launcher = inputs.zeros ? kernels::cutlass_kernels::genericMoeGemmKernelLauncher<
-                                    T, WeightType, GemmOutputType, Arch,
-                                    cutlass::WeightOnlyQuantOp::FINEGRAINED_SCALE_AND_ZEROS,
-                                    EpilogueTag, ThreadblockShape, WarpShape, Stages>::call
-                              : kernels::cutlass_kernels::genericMoeGemmKernelLauncher<
-                                    T, WeightType, GemmOutputType, Arch,
-                                    cutlass::WeightOnlyQuantOp::FINEGRAINED_SCALE_ONLY, EpilogueTag,
-                                    ThreadblockShape, WarpShape, Stages>::call;
+      launcher = inputs.zeros
+                     ? tensorrt_llm::kernels::cutlass_kernels_oss::genericMoeGemmKernelLauncher<
+                           T, WeightType, GemmOutputType, Arch,
+                           cutlass::WeightOnlyQuantOp::FINEGRAINED_SCALE_AND_ZEROS, EpilogueTag,
+                           ThreadblockShape, WarpShape, Stages>::call
+                     : tensorrt_llm::kernels::cutlass_kernels_oss::genericMoeGemmKernelLauncher<
+                           T, WeightType, GemmOutputType, Arch,
+                           cutlass::WeightOnlyQuantOp::FINEGRAINED_SCALE_ONLY, EpilogueTag,
+                           ThreadblockShape, WarpShape, Stages>::call;
     }
     launcher(inputs, sm_count_);
   } else {
@@ -519,17 +521,23 @@ void dispatchMoeGemmToCutlass(
   }
 }
 
+}  // namespace tensorrt_llm::kernels::cutlass_kernels_oss
+
+namespace tensorrt_llm::kernels::cutlass_kernels {
+
 template <typename T, typename WeightType, typename OutputType, typename ScaleBiasType>
 std::vector<cutlass_extensions::CutlassGemmConfig>
-MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::getConfigs() const {
-  return getConfigs(sm_);
+MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::getConfigs(
+    bool supports_finalize_fusion) const {
+  return getConfigs(sm_, supports_finalize_fusion);
 }
 
 template <typename T, typename WeightType, typename OutputType, typename ScaleBiasType>
 std::vector<cutlass_extensions::CutlassGemmConfig>
-MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::getConfigs(int sm) {
+MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::getConfigs(int sm,
+                                                                    bool supports_finalize_fusion) {
   std::vector<cutlass_extensions::CutlassGemmConfig> candidate_configs =
-      getTmaWarpSpecializedConfigs(sm);
+      getTmaWarpSpecializedConfigs(sm, supports_finalize_fusion);
   std::vector<cutlass_extensions::CutlassGemmConfig> ampere_configs = getAmpereConfigs(sm);
   std::copy(ampere_configs.begin(), ampere_configs.end(), std::back_inserter(candidate_configs));
   return candidate_configs;
@@ -552,19 +560,21 @@ MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::getAmpereConfigs(int sm
   auto config_type_param = static_cast<CutlassGemmConfig::CandidateConfigTypeParam>(
       weight_only_flag | simt_only_flag | grouped_gemm_flag | enable_hopper | fp8_only_flag);
 
-  if (!kernels::cutlass_kernels::isValidAmpereMOESpecialisation<T, WeightType>() ||
+  if (!tensorrt_llm::kernels::cutlass_kernels::isValidAmpereMOESpecialisation<T, WeightType>() ||
       (use_w4afp8 && sm != 89) || use_wfp4a16) {
     return {};
   }
 
   std::vector<cutlass_extensions::CutlassGemmConfig> ampere_configs =
-      kernels::cutlass_kernels::get_candidate_configs(sm, max_split_k, config_type_param);
+      tensorrt_llm::kernels::cutlass_kernels::get_candidate_configs(sm, max_split_k,
+                                                                    config_type_param);
   return ampere_configs;
 }
 
 template <typename T, typename WeightType, typename OutputType, typename ScaleBiasType>
 std::vector<cutlass_extensions::CutlassGemmConfig>
-MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::getTmaWarpSpecializedConfigs(int sm) {
+MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::getTmaWarpSpecializedConfigs(
+    int sm, bool supports_finalize_fusion) {
   using tensorrt_llm::cutlass_extensions::CutlassGemmConfig;
   static constexpr auto weight_only_flag =
       std::is_same<T, WeightType>::value ? CutlassGemmConfig::NONE : CutlassGemmConfig::WEIGHT_ONLY;
@@ -577,28 +587,32 @@ MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::getTmaWarpSpecializedCo
   static constexpr auto fp8_only_flag =
       use_fp8 ? CutlassGemmConfig::FP8_ONLY : CutlassGemmConfig::NONE;
   static constexpr auto fp4_only_flag =
-      (use_fp4 || use_wfp4afp4) ? CutlassGemmConfig::FP4_ONLY : CutlassGemmConfig::NONE;
+      (use_fp4 || use_wfp4afp8) ? CutlassGemmConfig::FP4_ONLY : CutlassGemmConfig::NONE;
+  static constexpr auto fp8fp4_mixed_flag =
+      use_wfp4afp8 ? CutlassGemmConfig::FP8FP4_MIXED : CutlassGemmConfig::NONE;
   auto config_type_param = static_cast<CutlassGemmConfig::CandidateConfigTypeParam>(
       weight_only_flag | simt_only_flag | grouped_gemm_flag | enable_blackwell | enable_hopper |
-      fp8_only_flag | fp4_only_flag);
+      fp8_only_flag | fp4_only_flag | fp8fp4_mixed_flag);
   TLLM_CHECK_WITH_INFO(!(enable_blackwell && enable_hopper),
                        "Blackwell and hopper flags are mutually exclusive");
 
+  sm = use_wfp4afp8 && sm == 103 ? 100 : sm;
   if (sm >= 100 && sm < 120 &&
-      !kernels::cutlass_kernels::isValidBlackwellMOESpecialisation<T, WeightType>()) {
+      !tensorrt_llm::kernels::cutlass_kernels::isValidBlackwellMOESpecialisation<T, WeightType>()) {
     TLLM_LOG_TRACE(
         "Blackwell is not supported for this configuration, not selecting any TMA WS "
         "implementations");
     return {};
   }
   if ((sm == 120 || sm == 121) &&
-      !kernels::cutlass_kernels::isValidSM120MOESpecialisation<T, WeightType>()) {
+      !tensorrt_llm::kernels::cutlass_kernels::isValidSM120MOESpecialisation<T, WeightType>()) {
     TLLM_LOG_TRACE(
         "Blackwell SM120 is not supported for this configuration, not selecting any TMA WS "
         "implementations");
     return {};
   }
-  if (enable_hopper && !kernels::cutlass_kernels::isValidHopperMOESpecialisation<T, WeightType>()) {
+  if (enable_hopper &&
+      !tensorrt_llm::kernels::cutlass_kernels::isValidHopperMOESpecialisation<T, WeightType>()) {
     TLLM_LOG_TRACE(
         "Hopper is not supported for this configuration, not selecting any TMA WS implementations");
     return {};
@@ -606,6 +620,51 @@ MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::getTmaWarpSpecializedCo
 
   std::vector<cutlass_extensions::CutlassGemmConfig> tma_ws_configs =
       kernels::cutlass_kernels::get_candidate_configs(sm, max_split_k, config_type_param);
+
+  if (sm == 103 && use_fp4) {
+    // Explicitly select SM100 as well
+    auto sm100_configs = tensorrt_llm::kernels::cutlass_kernels::get_candidate_configs(
+        100, max_split_k, config_type_param);
+    std::copy(sm100_configs.begin(), sm100_configs.end(), std::back_inserter(tma_ws_configs));
+  }
+
+  if (supports_finalize_fusion) {
+    // Duplicate the configs and set the epilogue fusion type to FINALIZE
+    auto finalize_configs = tma_ws_configs;
+    std::transform(finalize_configs.begin(), finalize_configs.end(),
+                   std::back_inserter(tma_ws_configs), [](auto& config) {
+                     config.epilogue_fusion_type =
+                         cutlass_extensions::CutlassGemmConfig::EpilogueFusionType::FINALIZE;
+                     return config;
+                   });
+
+    // Finalize fusion is only supported for TMA epilogue schedule
+    tma_ws_configs.erase(
+        std::remove_if(
+            tma_ws_configs.begin(), tma_ws_configs.end(),
+            [](auto& config) {
+              return config.epilogue_fusion_type ==
+                         cutlass_extensions::CutlassGemmConfig::EpilogueFusionType::FINALIZE &&
+                     config.epilogue_schedule == cutlass_extensions::EpilogueScheduleType::NO_SMEM;
+            }),
+        tma_ws_configs.end());
+  }
+
+  auto swap_ab_configs = tma_ws_configs;
+  std::transform(swap_ab_configs.begin(), swap_ab_configs.end(), std::back_inserter(tma_ws_configs),
+                 [](auto& config) {
+                   TLLM_CHECK_WITH_INFO(!config.swap_ab, "Swap AB is already set");
+                   config.swap_ab = true;
+                   return config;
+                 });
+
+  if (use_w4_groupwise) {
+    // w4 groupwise implementation requires swap_ab to be true
+    tma_ws_configs.erase(std::remove_if(tma_ws_configs.begin(), tma_ws_configs.end(),
+                                        [](auto& config) { return !config.swap_ab; }),
+                         tma_ws_configs.end());
+  }
+
   return tma_ws_configs;
 }
 
@@ -617,12 +676,15 @@ bool MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::isTmaWarpSpecializ
 }
 
 template <typename T, typename WeightType, typename OutputType, typename ScaleBiasType>
-bool MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::supportsTmaWarpSpecialized() const {
-  return (sm_ == 90 && kernels::cutlass_kernels::isValidHopperMOESpecialisation<T, WeightType>()) ||
-         (sm_ >= 100 && sm_ < 120 &&
-          kernels::cutlass_kernels::isValidBlackwellMOESpecialisation<T, WeightType>()) ||
-         ((sm_ == 120 || sm_ == 121) &&
-          kernels::cutlass_kernels::isValidSM120MOESpecialisation<T, WeightType>());
+bool MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::supportsTmaWarpSpecialized(int sm) {
+  return (sm == 90 &&
+          tensorrt_llm::kernels::cutlass_kernels::isValidHopperMOESpecialisation<T,
+                                                                                 WeightType>()) ||
+         (sm >= 100 && sm < 120 &&
+          tensorrt_llm::kernels::cutlass_kernels::isValidBlackwellMOESpecialisation<
+              T, WeightType>()) ||
+         ((sm == 120 || sm == 121) &&
+          tensorrt_llm::kernels::cutlass_kernels::isValidSM120MOESpecialisation<T, WeightType>());
 }
 
 template <typename T, typename WeightType, typename OutputType, typename ScaleBiasType>
@@ -677,63 +739,64 @@ void MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::dispatchToArch(
                        "Hopper configuration provided for non-Hopper architecture");
 
   if (sm_ >= 75 && sm_ < 80) {
-#ifdef ENABLE_FP4
-    if constexpr (!std::is_same_v<WeightType, __nv_fp4_e2m1>) {
-      dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType, cutlass::arch::Sm75, EpilogueTag>(
+#if defined(ENABLE_FP4)
+    constexpr bool is_fp4 = std::is_same_v<WeightType, __nv_fp4_e2m1>;
+#else
+    constexpr bool is_fp4 = false;
+#endif
+    if constexpr (!is_fp4) {
+      cutlass_kernels_oss::dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType,
+                                                    cutlass::arch::Sm75, EpilogueTag>(
           inputs, multi_processor_count_);
     } else {
       TLLM_THROW("FP4 data type is not supported on SM < 90");
     }
-#else
-    TLLM_THROW("FP4 data type is not supported on SM < 90");
-#endif
   } else if (sm_ >= 80 && sm_ < 90) {
-    if constexpr (use_fp8 || use_w4afp8) {
-#if defined(ENABLE_FP8)
-      static_assert(
-          !std::is_same_v<OutputType, __nv_fp8_e4m3> && !std::is_same_v<OutputType, __nv_fp8_e5m2>,
-          "FP8 GEMM Output not supported");
+#if defined(ENABLE_FP4)
+    constexpr bool is_fp4 = std::is_same_v<WeightType, __nv_fp4_e2m1>;
+#else
+    constexpr bool is_fp4 = false;
 #endif
-      TLLM_CHECK_WITH_INFO(sm_ == 89, "For sm >= 80 and < 90, fp8 is only supported with sm == 89");
-      dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType, cutlass::arch::Sm89, EpilogueTag>(
-          inputs, multi_processor_count_);
-    } else {
-#ifdef ENABLE_FP4
-      if constexpr (std::is_same_v<WeightType, __nv_fp4_e2m1>) {
-        TLLM_THROW("FP4 data type is not supported on SM < 90");
+    if constexpr (!is_fp4) {
+      if constexpr (use_fp8 || use_w4afp8) {
+#if defined(ENABLE_FP8)
+        static_assert(!std::is_same_v<OutputType, __nv_fp8_e4m3> &&
+                          !std::is_same_v<OutputType, __nv_fp8_e5m2>,
+                      "FP8 GEMM Output not supported");
+#endif
+
+        TLLM_CHECK_WITH_INFO(sm_ == 89,
+                             "For sm >= 80 and < 90, fp8 is only supported with sm == 89");
+        cutlass_kernels_oss::dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType,
+                                                      cutlass::arch::Sm89, EpilogueTag>(
+            inputs, multi_processor_count_);
       } else {
-        dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType, cutlass::arch::Sm80, EpilogueTag>(
+        cutlass_kernels_oss::dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType,
+                                                      cutlass::arch::Sm80, EpilogueTag>(
             inputs, multi_processor_count_);
       }
-#else
-      dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType, cutlass::arch::Sm80, EpilogueTag>(
-          inputs, multi_processor_count_);
-#endif
+    } else {
+      TLLM_THROW("FP4 data type is not supported on SM < 90");
     }
   } else if (sm_ >= 90) {
-    // For SM120+ FP8 MoE, redirect to SM89 (Ada) FP8 kernel implementations.
-    if constexpr (use_fp8) {
+    // For SM120+ pure FP8 MoE (not FP8 x FP4), redirect to SM89 (Ada) FP8 kernel implementations.
+    if constexpr (use_fp8 && !use_wfp4afp8) {
       if (sm_ >= 120) {
-        dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType, cutlass::arch::Sm89, EpilogueTag>(
+        cutlass_kernels_oss::dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType,
+                                                      cutlass::arch::Sm89, EpilogueTag>(
             inputs, multi_processor_count_);
         return;
       }
     }
 
-    if constexpr (kernels::cutlass_kernels::isValidTmaWarpSpecializedMOESpecialisation<
-                      T, WeightType, EpilogueTag>() &&
+    if constexpr (tensorrt_llm::kernels::cutlass_kernels::
+                      isValidTmaWarpSpecializedMOESpecialisation<T, WeightType, EpilogueTag>() &&
                   !use_w4_groupwise) {
       // We allow both tma warp specialized and SM80 configurations to coexist because for some
       // cases with small numbers of tokens SM80 is faster. We check here to see which is selected
       if (inputs.gemm_config.sm_version >= 90) {
-        bool is_same_sm = inputs.gemm_config.sm_version == sm_;
-        // gemm_config.sm_version indicates the kernel pipeline, which is always 100 for 100, 103,
-        // 110 below logging helps confirming the cutlass pipeline matches the device major version
-        bool is_sm110 = inputs.gemm_config.sm_version == 100 && sm_ == 110;
-        bool is_sm103 = inputs.gemm_config.sm_version == 100 && sm_ == 103;
-        // SM120 and SM121 are architecturally identical
-        bool is_sm120 = (inputs.gemm_config.sm_version == 120) && (sm_ == 120 || sm_ == 121);
-        TLLM_CHECK_WITH_INFO(is_same_sm || is_sm110 || is_sm103 || is_sm120,
+        // Check the major version of the SM matches
+        TLLM_CHECK_WITH_INFO(inputs.gemm_config.sm_version / 10 == sm_ / 10,
                              "Using SM %d configuration for SM %d device",
                              inputs.gemm_config.sm_version, sm_);
         TLLM_CHECK_WITH_INFO(inputs.biases != nullptr || hopper_inputs.ptr_c == nullptr,
@@ -746,11 +809,11 @@ void MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::dispatchToArch(
         auto select_function = [&]() {
           switch (hopper_inputs.fusion) {
             case TmaWarpSpecializedGroupedGemmInput::EpilogueFusion::FINALIZE:
-              return &dispatchMoeGemmSelectTileShapeTmaWarpSpecialized<
+              return &cutlass_kernels_oss::dispatchMoeGemmSelectTileShapeTmaWarpSpecialized<
                   T, WeightType, OutputType, EpilogueTag,
                   TmaWarpSpecializedGroupedGemmInput::EpilogueFusion::FINALIZE>;
             case TmaWarpSpecializedGroupedGemmInput::EpilogueFusion::NONE:
-              return &dispatchMoeGemmSelectTileShapeTmaWarpSpecialized<
+              return &cutlass_kernels_oss::dispatchMoeGemmSelectTileShapeTmaWarpSpecialized<
                   T, WeightType, OutputType, EpilogueTag,
                   TmaWarpSpecializedGroupedGemmInput::EpilogueFusion::NONE>;
             case TmaWarpSpecializedGroupedGemmInput::EpilogueFusion::ACTIVATION:
@@ -775,16 +838,16 @@ void MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::dispatchToArch(
                            "w4afp8 is only supported for TMA warp specialization");
       // EpilogueTag is ignored
       if (inputs.k % 512 == 0) {
-        sm90_dispatch_moe_mixed_dtype_gemm_to_cutlass<T, WeightType, ScaleBiasType,
-                                                      cutlass_extensions::EpilogueOpDefault, 4>(
+        cutlass_kernels_oss::sm90_dispatch_moe_mixed_dtype_gemm_to_cutlass<
+            T, WeightType, ScaleBiasType, cutlass_extensions::EpilogueOpDefault, 4>(
             inputs, hopper_inputs, multi_processor_count_, nullptr);
       } else if (inputs.k % 256 == 0) {
-        sm90_dispatch_moe_mixed_dtype_gemm_to_cutlass<T, WeightType, ScaleBiasType,
-                                                      cutlass_extensions::EpilogueOpDefault, 2>(
+        cutlass_kernels_oss::sm90_dispatch_moe_mixed_dtype_gemm_to_cutlass<
+            T, WeightType, ScaleBiasType, cutlass_extensions::EpilogueOpDefault, 2>(
             inputs, hopper_inputs, multi_processor_count_, nullptr);
       } else if (inputs.k % 128 == 0) {
-        sm90_dispatch_moe_mixed_dtype_gemm_to_cutlass<T, WeightType, ScaleBiasType,
-                                                      cutlass_extensions::EpilogueOpDefault, 1>(
+        cutlass_kernels_oss::sm90_dispatch_moe_mixed_dtype_gemm_to_cutlass<
+            T, WeightType, ScaleBiasType, cutlass_extensions::EpilogueOpDefault, 1>(
             inputs, hopper_inputs, multi_processor_count_, nullptr);
       } else {
         TLLM_THROW("Invalid GEMM K size %d", (int)inputs.k);
@@ -796,16 +859,16 @@ void MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::dispatchToArch(
       TLLM_CHECK_WITH_INFO(inputs.gemm_config.is_tma_warp_specialized,
                            "wfp4a16 is only supported for TMA warp specialization");
       // EpilogueTag is ignored
-      sm90_dispatch_moe_mixed_dtype_gemm_to_cutlass<T, WeightType, ScaleBiasType,
-                                                    cutlass_extensions::EpilogueOpDefault, 1>(
+      cutlass_kernels_oss::sm90_dispatch_moe_mixed_dtype_gemm_to_cutlass<
+          T, WeightType, ScaleBiasType, cutlass_extensions::EpilogueOpDefault, 1>(
           inputs, hopper_inputs, multi_processor_count_, nullptr);
       return;
     }
 #endif
 
     // Do Ampere case instead
-    if constexpr (kernels::cutlass_kernels::isValidAmpereMOESpecialisation<T, WeightType,
-                                                                           EpilogueTag>()) {
+    if constexpr (tensorrt_llm::kernels::cutlass_kernels::isValidAmpereMOESpecialisation<
+                      T, WeightType, EpilogueTag>()) {
       TLLM_CHECK_WITH_INFO(!use_fp8, "No fallback FP8 implementation available");
       TLLM_CHECK_WITH_INFO(use_w4afp8 || !hopper_inputs.isValid(),
                            "Non-specialized Hopper implementation is being rerouted to fallback "
@@ -818,10 +881,12 @@ void MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::dispatchToArch(
                            "Using SM %d configuration for SM80 fallback implementation",
                            inputs.gemm_config.sm_version);
       if constexpr (use_fp8) {
-        dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType, cutlass::arch::Sm89, EpilogueTag>(
+        cutlass_kernels_oss::dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType,
+                                                      cutlass::arch::Sm89, EpilogueTag>(
             inputs, multi_processor_count_);
       } else {
-        dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType, cutlass::arch::Sm80, EpilogueTag>(
+        cutlass_kernels_oss::dispatchMoeGemmToCutlass<T, WeightType, ScaleBiasType,
+                                                      cutlass::arch::Sm80, EpilogueTag>(
             inputs, multi_processor_count_);
       }
     } else {
@@ -848,18 +913,21 @@ template <typename T, typename WeightType, typename OutputType, typename ScaleBi
 size_t MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::calcMaxWorkspaceSize(
     int num_experts) const {
   if constexpr (use_w4_groupwise) {
-    return calcMaxWorkspaceSizeTmaWarpSpecializedMixedInput<T, WeightType, OutputType>(
+    return cutlass_kernels_oss::calcMaxWorkspaceSizeTmaWarpSpecializedMixedInput<T, WeightType,
+                                                                                 OutputType>(
         num_experts, multi_processor_count_);
   }
   if (!supportsTmaWarpSpecialized()) {
     return 0;
   }
-  if constexpr (kernels::cutlass_kernels::isValidTmaWarpSpecializedMOESpecialisation<
+  if constexpr (tensorrt_llm::kernels::cutlass_kernels::isValidTmaWarpSpecializedMOESpecialisation<
                     T, WeightType>() &&
                 !use_w4afp8 && !use_wfp4a16) {
-    auto configs = getTmaWarpSpecializedConfigs(sm_);
+    // Finalize fusion may not actually be supported by the kernel,
+    // if they are not we will catch the error and skip them
+    auto configs = getTmaWarpSpecializedConfigs(sm_, true);
     auto fpX_block_scaling_type = TmaWarpSpecializedGroupedGemmInput::FpXBlockScalingType::NONE;
-    if constexpr (use_wfp4afp4) {
+    if constexpr (use_wfp4afp8) {
       fpX_block_scaling_type = TmaWarpSpecializedGroupedGemmInput::FpXBlockScalingType::MXFPX;
     } else if (use_fp4) {
       fpX_block_scaling_type = TmaWarpSpecializedGroupedGemmInput::FpXBlockScalingType::NVFP4;
@@ -867,17 +935,19 @@ size_t MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::calcMaxWorkspace
     size_t max_size = 0;
     bool has_config = false;
     for (auto conf : configs) {
-#define CALC_SIZE_FUSION(FUSION)                                                               \
-  do {                                                                                         \
-    try {                                                                                      \
-      size_t size = calcMaxWorkspaceSizeTmaWarpSpecialized<T, WeightType, OutputType, FUSION>( \
-          num_experts, conf, multi_processor_count_, fpX_block_scaling_type);                  \
-      max_size = std::max(max_size, size);                                                     \
-      has_config = true;                                                                       \
-    } catch (tensorrt_llm::common::TllmException const& e) {                                   \
-      TLLM_LOG_TRACE("Unsupported config skipped when calculating MOE workspace size %s",      \
-                     e.what());                                                                \
-    }                                                                                          \
+#define CALC_SIZE_FUSION(FUSION)                                                                 \
+  do {                                                                                           \
+    try {                                                                                        \
+      size_t size =                                                                              \
+          cutlass_kernels_oss::calcMaxWorkspaceSizeTmaWarpSpecialized<T, WeightType, OutputType, \
+                                                                      FUSION>(                   \
+              num_experts, conf, multi_processor_count_, fpX_block_scaling_type);                \
+      max_size = std::max(max_size, size);                                                       \
+      has_config = true;                                                                         \
+    } catch (tensorrt_llm::common::TllmException const& e) {                                     \
+      TLLM_LOG_TRACE("Unsupported config skipped when calculating MOE workspace size %s",        \
+                     e.what());                                                                  \
+    }                                                                                            \
   } while (0)
 
       CALC_SIZE_FUSION(TmaWarpSpecializedGroupedGemmInput::EpilogueFusion::NONE);
@@ -927,9 +997,6 @@ void MoeGemmRunner<T, WeightType, OutputType, ScaleBiasType>::moeGemmBiasAct(
     case ActivationType::Geglu:
       runGemm<cutlass_extensions::EpilogueOpDefaultFtGelu>(inputs, hopper_inputs);
       break;
-    case ActivationType::Relu2:
-      TLLM_THROW("Relu2 is not supported.");
-      break;
     case ActivationType::InvalidType:
       TLLM_THROW("Activation type for fpA_intB must be valid.");
       break;
