@@ -126,8 +126,9 @@ class Runner {
            int32_t* expandedIdxToPermutedIdx, int32_t* permutedIdxToExpandedIdx,
            int32_t* permutedIdxToTokenIdx, void* expertWeights, int32_t* numTokensPerExpert,
            int32_t* ctaIdxXyToBatchIdx, int32_t* ctaIdxXyToMnLimit, int32_t* numNonExitingCtas,
-           batchedGemm::trtllm::gen::Dtype dtypeElt, bool useRoutingScalesOnInput,
-           bool useDeepSeekFp8, RoutingMethodType routingMethodType, cudaStream_t stream);
+           batchedGemm::trtllm::gen::Dtype dtypeElt, batchedGemm::trtllm::gen::Dtype dtypeBias,
+           bool useRoutingScalesOnInput, bool useDeepSeekFp8, RoutingMethodType routingMethodType,
+           cudaStream_t stream);
 
  private:
   int32_t mTileTokensDim{8};
@@ -304,7 +305,11 @@ struct MoEWorkspace {
   int32_t* expanded_idx_to_permuted_idx = nullptr;
   int32_t* permuted_idx_to_expanded_idx = nullptr;
   int32_t* permuted_idx_to_token_idx = nullptr;
+
+  // consumed by finalize kernel
   void* expert_weights = nullptr;  // [num_tokens, top_k] in bfloat16 = mDtypeExpW
+  // consumed by permuteGemm1 kernel
+  void* token_scales = nullptr;
 
   int32_t* cta_idx_xy_to_batch_idx = nullptr;
   int32_t* cta_idx_xy_to_mn_limit = nullptr;
